local db

function flsys:InitializeComm()
  self:RegisterComm("flamingsysdata") -- for syncing data between mls
  self:RegisterEvent("CHAT_MSG_ADDON")
  RegisterAddonMessagePrefix("flamingsys")
  print('comm initializing')
  db = flsys.db
end

function flsys:SendStandings(sender, uptodate)
  if uptodate then
    SendAddonMessage("flamingloot", "uptodate:", "WHISPER", sender)
  else
    sstr =""
    for i, name in pairs(db.sortlist) do
      sstr = sstr..name..": "
      if db.mode ~= 2.2 then
        sstr = sstr..db.players[name].current_pr.."\n"
      else
        sstr = sstr..db.players[name].gp_total.."\n"
      end
    end
    if sender=="raid" then
      SendAddonMessage("flamingloot", "standings:"..sstr, "RAID")
      db.lastbroadcast = time()
    else
      SendAddonMessage("flamingloot", "standings:"..sstr, "WHISPER", sender)
    end
  end
end

function flsys:SendVersion()
  local vers = GetAddOnMetadata("flamingloot", "Version")
  local one, two = string.match(vers, "(%d).*(%d)")
  if one and two then
    SendAddonMessage("flamingloot", "version:"..one..two, "RAID")
    flsys:Debug(3, "VERSION", "Sent check to raid: version:"..one..two)
  end
end

function flsys:CHAT_MSG_ADDON(event, prefix, message, channel, sender)
  if prefix=="flamingsys" then
    local command, value = string.match(message, "(%a+):*(.*)")
    if command=="datareply" then
      flsys:Print(sender.." "..value)
    elseif command=="standings" then
      flsys:SendStandings(sender, (tonumber(value))<db.lastbroadcast)
    end
  end
end

function flsys:OnCommReceived(prefix, message, channel, sender)
  if prefix == "flamingsysdata" then
    local command, value = string.match(message, "(%a+):*(.*)")
    if command == "data" then
      local dialog = StaticPopup_Show("RECEIVE_DATA", sender)
      if dialog then
        dialog.text = value
        dialog.player = sender
      end
    end
  end
end

---- TODO where to put these
-------------------------------------------------------------------------------
--- CHAT COMMANDS
-------------------------------------------------------------------------------
function flsys:ProcessChatCmd(input)
  if not input or input:trim() == "" then
    flsys:Print("Addon commands:\n|cffffff78/fss|r = View the main panel.\n|cffffff78/fs opt|r = View the options window. Also contains a help panel.\n|cffffff78/fs gp [ilvl]|r = View a printout of GP values for an ilvl.\n|cffffff78/fs decay|r = Decay ep and gp (default setting will prompt after raids so you don't need to use this).\n|cffffff78/loot|r = (during a raid only) View a cache of the most recent mob's drops and all the rolls made on them.  Only available after the rolling period ends and only for the most recent mob.")
  elseif input=="opt" then
    if not flopt:IsEnabled() then
      fld:Enable()
      flopt:Enable()
    end
    InterfaceOptionsFrame_OpenToCategory("flaming loot ml")
  elseif input=="decay" then
    if not db.inprogress then
      StaticPopup_Show("DECAY")
    else
      StaticPopup_Show("NOTDURINGRAID")
    end
  else
    local ilvl = tonumber(string.match(input, "gp (%d+)"))
    if ilvl then
      if (-100 < ilvl - db.standard_ilvl) and (ilvl - db.standard_ilvl < 100) then --reasonable-ish number
        flsys:PrintGPValues(ilvl)
      else
        flsys:Print("Not a valid item level. Try again.")
      end
    else
      flsys:Print("Not a valid command. Type /fs to view a list of commands")
    end
  end
end

function flsys:PrintGPValues(ilvl)
  local mult = flsys:Round((2^((ilvl-db.standard_ilvl)/26)), 3)
  flsys:Print(ilvl.." values:\n2H weapons, hunter ranged: "..(2000*mult).."\nCaster/healer main hand, tank shield: "..(1500*mult).."\nDual wield weapon, chest, helm, legs, trinket: "..(1000*mult).."\nShoulder, belt, boots, gloves: "..(750*mult).."\nOff hand, tank mainhand, neck, ring, cloak, bracers: "..(500*mult))
end

