
function flsys:StartRaid(raidname)
  local mlid = select(3, GetLootMethod())
  if flsys.db.inprogress then
    flsys:Print('You already have a raid in progress.')
  elseif not IsInRaid() then
    flsys:Print('You are not in a raid.')
  elseif not mlid then
    flsys:Print('You must be the masterlooter to start a raid.')
  elseif select(1, GetRaidRosterInfo(mlid)) ~= GetUnitName('player') then
    flsys:Print('You must be the masterlooter to start a raid.')
  else --prereqs met; start the raid
    fld:Enable()
    flsys:RegisterEvent('GROUP_ROSTER_UPDATE')
    flsys.db.inprogress = raidname or 'Raid'
    flsys.ui:MainButtons()
    table.insert(flsys.db.dates, 1, date('%m/%d %H:%M'))
    flsys:Log(0, 'start', 'all', flsys.db.inprogress..' started', 0)  --FIX
    flsys:UpdateRaidRoster()
    fld:StartRaid()

    flsys:SendVersion()

    if flsys.db.mode == 1 then
      if flsys.db.promptrecur then
        StaticPopup_Show('START_RECUR')
      end
    elseif flsys.db.mode == 2.1 then
      flsys:MassEP(4000, 'single, GP')
    elseif flsys.db.mode == 2.2 then
      flsys:MassEP(3, 'single, count')
    end
  end
end

-- Frame used as timer
local recur_frame = CreateFrame('Frame', 'flsys_TimerFrame')
recur_frame:SetScript('OnUpdate', function(self, elapsed) flsys:CheckTime(self, elapsed) end)
recur_frame:Hide()

function flsys:ResumeRaid()
  flsys:UpdateRaidRoster()
  if flsys.db.next_award > 0 then
    flsys:Print('Recurring EP award resumed.')
    recur_frame:Show()
  end
  flsys:RegisterEvent('GROUP_ROSTER_UPDATE')
  fld:Enable()
  fld:StartRaid()
end

function flsys:EndRaid()
  if flsys.db.inprogress then
    flsys:Log(0, 'end', 'all', 'Raid ended') --TODO customize which raid
    flsys:Print('Raid ended.')
    flsys.db.dates[1] = flsys.db.dates[1]..' - '..date('%m/%d %H:%M')..' '..flsys.db.inprogress
    table.insert(flsys.db.dates, 1, '')
    flsys.db.inprogress = false
    for name, ntable in pairs(flsys.db.players) do
      if flsys.db.players[name].temp or flsys.db.players[name].alt then
        flsys:Debug(3, 'Removing', name, 'from system (alt or sub)')
        flsys:RemovePlayer(name)
      else
        flsys.db.players[name].inraid = nil
      end
    end
    flsys:EndRecur()
    flsys:UnregisterEvent('GROUP_ROSTER_UPDATE')
    flsys:UnregisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
    flsys:UnregisterEvent('TRADE_ACCEPT_UPDATE')
    fld:Disable()
    flsys:RefreshSortList()
    flsys.ui:MainButtons()
    if flsys.db.mode == 1 then
      flsys:DecayRaid()
    end
  else
    flsys:Print('You do not have a raid in progress.')
  end
end


function flsys:StartRecur(minutes) --TODO should do this check before the popup
  if flsys.db.next_award > 0 then
    flsys:Print('You already have a recurring award in progress.  /fs endrecur to end it.')
  elseif minutes >= 0 and minutes < 60 and flsys.db.inprogress then
    local start = GetTime() + minutes*60
    flsys.db.next_award = start
    flsys:Print('Recurring award starts in '..minutes..'min. '..flsys.db.ep_amt..' EP will be awarded every '..flsys.db.ep_recur..' min.')
    flsys:Debug(3, 'Recurring award starts in '..minutes..'min. '..flsys.db.ep_amt..' EP will be awarded every '..flsys.db.ep_recur..' min.')
    recur_frame:Show()
  elseif not flsys.db.sortlist[1] then --TODO why this check and not inprogress?
    flsys:Print('Recurring awards can only take place during an active raid.  /fs startraid to begin one.')
  else
    flsys:Print('Invalid entry.  Type /fs startrecur to bring up the window again.  Enter a number between 0 and 60.')
  end
end

function flsys:EndRecur()
  flsys.db.next_award = 0
  flsys:Debug(3, 'Ending recurring')
  recur_frame:Hide()
end

local interval = 0
local currtime
function flsys:CheckTime(self, elapsed)
  --- adapted from the frame/func in epgp_recurring
  -- called by: recur_frame.OnUpdate
  currtime = GetTime()
  interval = interval + elapsed
  if interval > 30 then
    interval = 0
    if currtime > flsys.db.next_award then
      flsys:MassEP(flsys.db.ep_amt, flsys.db.inprogress)
      flsys.db.next_award = flsys.db.next_award + flsys.db.ep_recur*60
    end
  end
end


-- After login, check if player is in raid. Retry because sometimes OnEnabled
-- fires before raid group can be detected

local delay_fra = CreateFrame('Frame', 'flsys_DelayTimerFrame')
local raid_check_timer = 0

function flsys:InRaidCheck(first_try)
  -- after login, if flsys.db.inprogress is true
  if IsInRaid() then
    StaticPopup_Show('RESUME_RAID')
  elseif first_try then
    delay_fra:SetScript('OnUpdate', function(self, elapsed) flsys:DelayRaidCheck(self, elapsed) end)
    delay_fra:Show()
  else
    flsys:EndRaid()
  end
end

function flsys:DelayRaidCheck(self, elapsed)
  raid_check_timer = raid_check_timer + elapsed
  if raid_check_timer > 2 then
    flsys:InRaidCheck(false)
    delay_fra:Hide()
  end
end

