--- Handles log frame, including popup export window

local ui = flsys.ui

local log_rows = {} -- log row buttons

function ui:InitializeLog()
  FLLogBox:SetPoint('TOPRIGHT', FLLogContainer, 'TOPRIGHT', -4, -17)
  FLLogBox:SetScript('OnVerticalScroll', function(self, offset)
    ui:LogScrolled(self, offset, 18)
  end)
  FLLogItemAmount:SetNumeric(Enable)
  FLLogAdd:SetScale(.8)
  FLLogExport:SetScript('OnClick', function() ui:ShowExportFrame('gear') end)

  FLExport.TitleText:SetText('Copy and paste text to spreadsheet')
  FLExport.TitleText:SetJustifyH('LEFT')
end


function ui:ShowLogFrame()
  if #log_rows == 0 then --create frames if needed
    local fontstr
    for i = 1, 13 do
      log_rows[i] = CreateFrame('Button', 'LogRow'..i, FLLogContainer, 'LogRowTemplate')
      log_rows[i]:SetPoint('TOPLEFT', FLLogContainer, 'TOPLEFT', 5, -18*i + 15)
      log_rows[i]:SetMotionScriptsWhileDisabled(true)
      fontstr = log_rows[i]:GetFontString()
      fontstr:SetJustifyH('LEFT')
    end
    -- scroll down to newest content
    flsys:Debug(3, 'LOG DISPLAY', 'created rows, calling on vert scroll')
    local starting_os = math.max(#flsys.db.epgplog - 13, 0)
    ui:UpdateLogFrame(starting_os)
    FLLog:Show()
    FLLogBoxScrollBar:SetValue(starting_os*18)
  elseif not FLLog:IsShown() then
    ui:UpdateLogFrame()
    FLLog:Show()
  end
end

function ui:UpdateLogFrame(offset)
  --- given the position of the scroll, set each log row's text and onclick
  -- @offset: given in number of rows. calculated by OnVerticalScroll
  -- calls: FauxScrollFrame_Update
  -- called by: ShowLogFrame, OnVerticalScroll

  local num_logs = #flsys.db.epgplog
  local start_raid_i = 1 --for decay calculations. 1 = current
  if num_logs > 0 then
    for i = 1, num_logs - (offset or 0) do
      if flsys.db.epgplog[i][3] == 'decay' then
        start_raid_i = start_raid_i + 1
      end
    end
  end
  for i = 1, 13 do
    local log_i = num_logs - (i-1) - (offset or 0)
    local raid_i = start_raid_i -- so on script don't all call the last version
    if log_i > 0 then
      local logitem = flsys.db.epgplog[log_i]
      log_rows[i]:SetText(flsys:LogItemtoString(logitem))
      fs = log_rows[i]:GetFontString()
      if fs:IsTruncated() then
        log_rows[i]:SetScript('OnEnter', function(self)
          GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
          GameTooltip:SetText(self:GetText())
          GameTooltip:Show()
        end)
        log_rows[i]:SetScript('OnLeave', function() GameTooltip:Hide() end)
      else
        log_rows[i]:SetScript('OnEnter', nil)
        log_rows[i]:SetScript('OnLeave', nil)
      end
      if logitem[3] == 'GP' or (logitem[3] == 'EP' and not string.find(logitem[4], ',')) then
        -- can edit/delete
        --flsys:Print(i..'gp or single ep, setting texture')
        log_rows[i]:Enable()
        log_rows[i]:SetNormalTexture('Interface\\Buttons\\UI-GuildButton-PublicNote-Up')
        log_rows[i]:SetPushedTexture('Interface\\Buttons\\UI-GuildButton-PublicNote-Up')
        log_rows[i]:SetScript('OnClick', function() ui:ShowEditLogItem(log_i, raid_i); end)
      elseif logitem[3] == 'start' then
        -- can add
        log_rows[i]:Enable()
        log_rows[i]:SetNormalTexture('Interface\\Buttons\\UI-AttributeButton-Encourage-Up')
        log_rows[i]:SetPushedTexture('Interface\\Buttons\\UI-AttributeButton-Encourage-Down')
        log_rows[i]:SetScript('OnClick', function() ui:ShowAddType(log_i, raid_i); end)
      else
        log_rows[i]:Disable()
        if logitem[3] == 'decay' then
          start_raid_i = start_raid_i - 1
        end
      end
      if not log_rows[i]:IsShown() then
        log_rows[i]:Show()
      end
    else
      -- extra row doesn't correspond to any log entry; hide it
      log_rows[i]:Hide()
    end
  end
  FauxScrollFrame_Update(FLLogBox, num_logs, 13, 18)
end

function ui:LogScrolled(self, value, itemHeight)
	FLLogBoxScrollBar:SetValue(value)
  offset = floor((value / itemHeight) + 0.5);
	self.offset = offset
  ui:UpdateLogFrame(offset)
end

function ui:LogChanged()

  -- update log frame if needed
  if FLLog:IsShown() then
    ui:UpdateLogFrame()
  end

  -- update log edit/add if needed
  if FLLogItem:IsShown() then
    local raid_i = FLLogItem:GetAttribute("raid")
    local log_i = FLLogItem:GetAttribute("log") + 1
    if logtype=="decay" then
      raid_i = raid_i + 1
    end
    flsys:ShowEditLogItem(log_i, raid_i)
  end

end

function ui:ShowAddType(log_i, raid_i)
  FLLogAddEP:SetScript('OnClick', function() ui:ShowAddToLog(log_i, raid_i, 'EP') end)
  FLLogAddGP:SetScript('OnClick', function() ui:ShowAddToLog(log_i, raid_i, 'GP') end)
  --FLLogAddMassEP:SetScript('OnClick', function() ui:ShowAddToLog(log_i, raid_i, 'EPmass') end)

  FLLogAdd:Show()
end

local function setup_dropdown(raid_i)
  -- used when adding to log or editing log item

  FLRosterDropDown:ClearAllPoints()
  FLRosterDropDown:SetParent(FLLogItem)
  FLRosterDropDown:SetPoint('BOTTOMLEFT', FLLogItem, 'BOTTOMLEFT', 70, 30)

  ui:InitializeAwardDropDown(raid_i == 1)
end


function ui:ShowAddToLog(log_i, raid_i, award_type)
  --- Shows FLLogItem, set up for adding an award retroactively
  -- @log_i: the index of the epgplog item where the raid started
  -- @raid_i: raid index, used to scale and place the award appropriately
  -- @award_type: ep or gp

  FLLogAdd:Hide()
  local start = flsys.db.epgplog[log_i]

  FLLogItem.TitleText:SetText('Adding award to '..date('%m/%d/%y', start[1])
                               ..' '..string.sub(start[5], 1, -8))
  FLLogItemAmount:SetNumber(0)
  FLLogItemType:SetText(award_type)
  FLLogItemReason:SetText('')
  FLLogItem:SetHeight(105)
  FLLogItemNote:SetText('')

  setup_dropdown(raid_i)

  FLLogItemDelete:Hide()
  FLLogItemUpdate:SetScript('OnClick', function() flsys:AlterLog(log_i, raid_i, 'add') end)
  FLLogItem:Show()
end


function ui:ShowEditLogItem(log_i, raid_i)
  --- Shows FLLogItem, set up for editing/deleting a log entry

  local logitem = flsys.db.epgplog[log_i]

  FLLogItem.TitleText:SetText('Editing award '..date('%m/%d/%y %H:%M', logitem[1]))
  FLLogItemAmount:SetNumber(logitem[2])
  FLLogItemType:SetText(logitem[3])
  FLLogItemReason:SetText(logitem[5])
  FLLogItem:SetAttribute('raid', raid_i) --TODO TEST
  FLLogItem:SetAttribute('log', log_i)

  setup_dropdown(raid_i)
  if flsys.db.players[logitem[4]] and (raid_i == 1 or not flsys.db.players[logitem[4]].temp) then
    FLLogItem:SetHeight(105)
    FLLogItemNote:SetText('')
    UIDropDownMenu_SetSelectedName(FLRosterDropDown, logitem[4])
  else
    FLLogItem:SetHeight(145)
    if alts_data[logitem[4]] and logtype[3] == 'GP' then
      -- EP awards for alts are stored under main's name, but GP are not
      FLLogItemNote:SetText('Note: This was a GP award to an alt, which has no impact on ongoing PR.')
    else
      FLLogItemNote:SetText('Note: This player is no longer in the database. Editing the amount or reason won\'t impact anyone\'s PR, but you can reassign the award to another player.')
    end
  end

  FLLogItemDelete:SetScript('OnClick', function()
    local dialog = StaticPopup_Show('DELETE_LOG_ITEM')
    if dialog then
      dialog.log_i = log_i
      dialog.raid_i = raid_i
    end
  end)
  FLLogItemDelete:Show()
  FLLogItemUpdate:SetScript('OnClick', function() flsys:AlterLog(log_i, raid_i, 'edit') end)
  FLLogItem:Show()
end



-- TODO can we guarantee sortlist is up to date?
function ui:ShowExportFrame(exporttype)
  -- epgplog format: 1 = timestamp, 2 = amount, 3 = type,
  --                 4 = player(s), 5 = reason/item, 6 = itemid
  -- spreadsheet order: 1 4 5 2 3 (6)
  local exporttext = ''

  if exporttype == 'gear' then
    for i, entry in ipairs(flsys.db.epgplog) do
      exporttext = exporttext..date('%m/%d/%y %H:%M', entry[1])..'\t'..entry[4]..'\t'..entry[5]..'\t'..entry[2]..'\t'..entry[3]
      if entry[6] then
        exporttext = exporttext..'\t'..entry[6]
      end
      exporttext = exporttext..'\n'
    end
    FLExportDone:SetScript('OnClick', function() FLExport:Hide(); flsys:ConfirmExportTime('epgp') end)

  elseif exporttype == 'standings' then
    local ddate = date('%m/%d/%y %H:%M', time())
    for i, name in ipairs(flsys.db.sortlist) do
      exporttext = exporttext..name..'\t'..flsys.db.players[name].ep_total..'\t'..flsys.db.players[name].gp_total..'\t'..flsys.db.players[name].current_pr..'\t'..ddate
      if i == 1 then
        exporttext = exporttext..'\t'..flsys:ExportToString()
      end
      exporttext = exporttext..'\n'
    end
    FLExportDone:SetScript('OnClick', function() FLExport:Hide(); end)
  end

  if exporttext ~= '' then
    FLExportText:SetText(exporttext)
    FLExport:Show()
  else
    self:Print('Nothing to export.')
  end
end

function ui:ConfirmExportTime(exporttype)
  -- Asks the player to confirm export complete so we can trim log
  dialog = StaticPopup_Show('EXPORT_COMPLETE', exporttype)
  if dialog then
    dialog.etype = exporttype
  end
end

