--- Handles UI for managing roster, not including standings-specific UI

local ui = flsys.ui
local db

function ui:InitializeRoster()
  db = flsys.db

  FLRosterFrame:SetScale(.8)
  FLRosterFrame.TitleText:SetText("Remove player")
  CreateFrame("Button", "FLRosterDropDown", FLRosterFrame, "UIDropDownMenuTemplate")
  FLRosterDropDown:SetPoint("LEFT")
end


function ui:AddPlayerEntered(done)
  --- Parse & validate input, create player or print error
  local name = flsys:Cap(FLAddPlayerPlayer:GetText())
  local ep = FLAddPlayerEP:GetNumber()
  local gp = FLAddPlayerGP:GetNumber()
  if name=="" then
    flsys:Print("Please enter a player name.")
  elseif db.players[name] and not db.players[name].temp then
    flsys:Print(name.." is already in the system.  If you want to reset their data, remove them and then add.")
  else
    flsys:Debug(3, "Adding player: "..name..", EP: "..ep..", GP: "..gp)
    flsys:NewPlayer(name, nil, nil, nil, ep, gp)
    FLAddPlayerPlayer:SetText("")
    FLAddPlayerEP:SetNumber(0)
    FLAddPlayerGP:SetNumber(0)
    if done then
      FLAddPlayer:Hide()
      flsys:RefreshSortList()
    end
  end
end

local function initialize_dropdown_all(self, level)
  -- used in raid for editing an award - shows all players even those not in db
  local button = {}
  for k,v in pairs(db.sortlist) do
    button.text = v
    button.func = function(self)
      UIDropDownMenu_SetSelectedName(FLRosterDropDown, self:GetText());
    end
    UIDropDownMenu_AddButton(button, level)
  end
end

local function initialize_dropdown_perm(self, level)
  -- shows only players that have been added to the system. used to remove players.
  local button = {}
  for k,v in pairs(db.sortlist) do
    if db.players[v] and not db.players[v].temp then
      button.text = v
      button.func = function(self)
        UIDropDownMenu_SetSelectedName(FLRosterDropDown, self:GetText());
      end
      UIDropDownMenu_AddButton(button, level)
    end
  end
end

function ui:InitializeAwardDropDown(current)
  if current then
    UIDropDownMenu_Initialize(FLRosterDropDown, initialize_dropdown_all)
  else
    UIDropDownMenu_Initialize(FLRosterDropDown, initialize_dropdown_perm)
  end
end

function ui:ShowRemovePlayer()
  if db.inprogress then
    StaticPopup_Show('NOTDURINGRAID')
  else
    InterfaceOptionsFrame:Hide()
    FLRosterDropDown:ClearAllPoints()
    FLRosterDropDown:SetParent(FLRosterFrame)
    FLRosterDropDown:SetPoint('CENTER')
    UIDropDownMenu_Initialize(FLRosterDropDown, initialize_dropdown_perm)
    FLRosterFrame:Show()
  end
end


