--- Handles the main window containing player standings

local ui = flsys.ui

local db
local rowbuttons = {}
local maindisplay = 1
local rosterpage = 0

-- initialize -> checkrows, mainbuttons
-- update -> update assumes there are correct # of rows, sets text ->
--   rosterpage
-- sys:refreshsortlist -> checkrows, updatepr
-- startraid -> mainbuttons, createroster
-- endraid -> refreshsortlist
-- 
function ui:InitializeStandings()
  -- store local ref to db
  db = flsys.db
  -- frame setup that wasn't done in the xml. only needs to be called once.
  FLMain:SetScale(.8)
  FLMainPrev:GetNormalTexture():SetRotation(-math.pi/2)
  FLMainPrev:SetScript("OnClick", function() ui:RosterPage(-1) end)
  FLMainNext:GetNormalTexture():SetRotation(math.pi/2)
  FLMainNext:SetScript("OnClick", function() ui:RosterPage(1) end)
  local x = 0
  local texts = {" ", "EP", "GP", "PR"}
  local widths = {77, 42, 42, 42}
  heading= CreateFrame("Button", "Heading" , FLMain)
  heading:SetSize(200,14)
  for i, w in ipairs(widths) do
    local c = FLMain:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    c:SetParent(Heading)
    c:SetWidth(w)
    c:SetJustifyH("LEFT")
    c:SetPoint("TOPLEFT", FLMain, "TOPLEFT", 15 + x, -42)
    c:SetText(texts[i])
    x = x + w
  end
  heading:Show()
  FLMainTitleText:SetText("         Standings") -- TODO move to xml
  SetPortraitToTexture(FLMainPortrait, "Interface\\Icons\\Achievement_Arena_5v5_6") --TODO move to xml
  FLMainButton3:SetScript("OnClick", function() ui:ShowLogFrame() end)
  -- state/data-dependent display
  ui:MainButtons()
  ui:RenderStandings()
end


-- CALL ON init, raid start, raid end
function ui:MainButtons()
  -- sets function of main buttons, depending on whether raid is in progress
  if not db.inprogress then
    FLMainButton1:SetText("Start")
    FLMainButton1:SetScript("OnClick", function() StaticPopup_Show("STARTRAID") end)
    FLMainButton2:SetText("Export")
    FLMainButton2:SetScript("OnClick", function() ui:ShowExportFrame("standings") end)
  else
    FLMainButton1:SetText("End")
    FLMainButton1:SetScript("OnClick", function() StaticPopup_Show("ENDRAID") end)
    FLMainButton2:SetText("Mass EP")
    FLMainButton2:SetScript("OnClick", function() ui:ShowMassEP() end)
  end
end

function ui:ShowMassEP()
  if db.inprogress then
    StaticPopup_Show('MASS_EP')
  else
    flsys:Print('You must have a raid in progress to do that.')
  end
end

function ui:HideMain()
  FLMain:Hide()
end


-- call on init, pr updated, player added/removed, raid update, reset data
-- (any change to roster or pr)
function ui:RenderStandings()
  --- update the player rows - text and click handling
  ui:CheckRows()
  for i, name in ipairs(db.sortlist) do
    local b = rowbuttons[i]
    if db.players[name].inraid then
      b.cells[1]:SetText("|cffCCFFFF"..name)
      b.cells[2]:SetText("|cffCCFFFF"..db.players[name].ep_total)
      b.cells[3]:SetText("|cffCCFFFF"..db.players[name].gp_total)
      b.cells[4]:SetText("|cffCCFFFF"..db.players[name].current_pr)
      if db.players[name].temp then
        b.cells[2]:SetText("|cffCCFFFF-")
        b.cells[4]:SetText("|cffCCFFFF-")
      end
    else
      b.cells[1]:SetText(name)
      b.cells[2]:SetText(db.players[name].ep_total)
      b.cells[3]:SetText(db.players[name].gp_total)
      b.cells[4]:SetText(db.players[name].current_pr)
    end
  end
  ui:RosterPage(0)
end


local function CreateTableRow(texttable)
  local widths = {77,42,42,42}
  local row = CreateFrame("Button", nil, FLMain)
  row:SetSize(200,14)
  row.cells = {}
  for i,w in ipairs(widths) do
    local c = row:CreateFontString(nil, "ARTWORK", "ChatFontSmall")
    c:SetJustifyH("LEFT")
    c:SetWidth(w)
    if #row.cells == 0 then
      c:SetPoint("LEFT", row, "LEFT", 0, 0)
    else
      c:SetPoint("LEFT", row.cells[#row.cells], "RIGHT", 0, 0)
    end
    c:SetText(texttable[i])
    table.insert(row.cells, c)
  end
  return row
end


-- call when sortlist size changes
function ui:CheckRows()
  ---creates & places row per sortlist if there isn't one already
  for i=1, #db.sortlist do
    if not rowbuttons[i] then
      local row = CreateTableRow({"-", "-", "-", "-"}, widths, FLMain)
      if i%13 ~= 0 then
        row:SetPoint("TOPLEFT", FLMain, "TOPLEFT", 18, -53-15*(i%13))
      else
        row:SetPoint("TOPLEFT", FLMain, "TOPLEFT", 18, -53-15*13)
      end
      rowbuttons[i] = row
    end
  end
end


function ui:RosterPage(pg)
  rosterpage = rosterpage + pg
  for i, button in ipairs(rowbuttons) do
    if i <= #db.sortlist and i > rosterpage * 13 and i <= (rosterpage + 1) * 13 then
      button:Show()
    else
      button:Hide()
    end
  end
  if rosterpage == 0 then
    FLMainPrev:Hide()
  else
    FLMainPrev:Show()
  end
  if (rosterpage + 1) * 13 >= #db.sortlist then
    FLMainNext:Hide()
  else
    FLMainNext:Show()
  end
end

